/*
 *
 * Universal variables application
 * Copyright (C) 2005, Andy Powell
 *
 *
 *  
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <asterisk/config.h>
#include <asterisk/file.h>
#include <asterisk/logger.h>
#include <asterisk/options.h>
#include <asterisk/channel.h>
#include <asterisk/pbx.h>
#include <asterisk/module.h>
#include <asterisk/cli.h>

#include <memcache.h>

#include <asterisk/caching.h>

static char *tdesc = "Sets/gets universal variables";

static char *app_setunivar = "SetUniVar";

static char *setunivar_synopsis = "SetUniVar(var=value)";

static char *setunivar_descrip =
"SetUniVar(var=value)\n"
"Sets a variable 'var' to 'value' over multiple servers.\n"
"\n\n"
"Universal variables are variables that exist over multiple "
" machines so data can be shared between machines in a 'cluster' "
" of installs."
"\n";

static char *app_getunivar = "GetUniVar";

static char *getunivar_synopsis = "GetUniVar(newvar=var)";

static char *getunivar_descrip =
"GetUniVar(newvar=var)\n"
"Gets a variable from multiple servers and places in in 'newvar'.\n"
"\n\n"
"Universal variables are variables that exist over multiple "
" machines so data can be shared between machines in a 'cluster' "
" of installs."
"\n";

static char *app_delunivar = "DelUniVar";

static char *delunivar_synopsis = "DelUniVar(var)";

static char *delunivar_descrip =
"DelUniVar(var)\n"
"Deletes a variable from multiple servers.\n"
"\n\n"
"Universal variables are variables that exist over multiple "
" machines so data can be shared between machines in a 'cluster' "
" of installs."
"\n";


static int allowreload=0;

STANDARD_LOCAL_USER;

LOCAL_USER_DECL;

static int getunivar_exec(struct ast_channel *chan, void *data)
{
	int res=0;
	struct localuser *u;
	char *s, *newvar=NULL, tmp[MAXRESULT];
	char *varn, *varn2;
	
	char *tmpres;
	char *tmpkey;
    
	LOCAL_USER_ADD(u);

	/* Check and parse arguments */
	if (data) {
		s = ast_strdupa((char *)data);
		if (s) {
			newvar = strsep(&s, "=");
			if (newvar && (newvar[0] != '\0')) {
				memset(tmp, 0, MAXRESULT);
	
				tmpres = malloc(MAX_VARLEN);
				if (!tmpres) {
					ast_log(LOG_ERROR, "Can't allocate key!\n");
					res = -1;
				} else {
					memset(tmpres, 0, sizeof(tmpres));
					get_cachevar(caching_servers,tmpres,s);
					pbx_builtin_setvar_helper(chan, newvar, tmpres);
					res = 0; 
					free(tmpres);
				}
			}				
		} else {
			ast_log(LOG_ERROR, "Out of memory\n");
			res = -1;
		}
	}

	LOCAL_USER_REMOVE(u);
	return res;
}

static int setunivar_exec(struct ast_channel *chan, void *data)
{
    int res=0;
    int modok=0;
    struct localuser *u;
    char *s, *newvar=NULL, tmp[MAXRESULT];
    char *varn, *varn2;

    char tmpvar[MAX_VARLEN];

    LOCAL_USER_ADD(u);
    
    /* Check and parse arguments */
    if (data) {
	        s = ast_strdupa((char *)data);
        	if (s) {
	            newvar = strsep(&s, "=");
        	    if (newvar && (newvar[0] != '\0')) {
	            	memset(tmpvar, 0, sizeof(tmpvar));

                	if (s) {
			        memset(tmpvar, 0, sizeof(tmpvar));
				snprintf(tmpvar,MAX_VARLEN-1,s);
				set_cachevar(caching_servers, newvar, s,0);
			}	
                    // set var
	            }
        	} else {
	            ast_log(LOG_ERROR, "Out of memory\n");
        	    res = -1;
	        }
    } 

    LOCAL_USER_REMOVE(u);
    return res;
}

static int delunivar_exec(struct ast_channel *chan, void *data)
{
    struct localuser *u;

    int res;
    char *s;
	

    LOCAL_USER_ADD(u);

    if (data) {
        s = ast_strdupa((char *)data);
        if (s) {
		res = del_cachevar(caching_servers, s);
	}

   }
   
   LOCAL_USER_REMOVE(u);
   return res;
}


static int handle_cli_univar_exec(int fd, int argc, char **argv)
{

        char *varval;
        int res = 0;

        if (argc < 2) {
                 ast_cli(fd, "\n\nToo few parameters!\n\nUseage:\n\tgetunivar varname\n\n");
                 return -1;
        }

	varval = malloc(MAX_VARLEN);

	if (!varval) {
		ast_cli(fd, "\n\nCouldn't allocate any memory.\n\n");
		return -1;
	}

	memset(varval, 0, sizeof(varval));
	if (!keyexists(caching_servers,argv[1])) {
		get_cachevar(caching_servers, varval, argv[1]);
		ast_cli(fd, "\n\nUniversal variable %s has a value of %s\n\n", argv[1], varval);
	} else {
		ast_cli(fd, "\n\nUniversal variable %s has no value, or does not exist\nCheck your case, variables are case sensitive\n\n", argv[1]);
	}
        

	free(varval);
        return res;
}

static char univar_usage[] = "Show a Universal variable.\n";

static struct ast_cli_entry cli_univar = {
        { "univar", NULL, NULL }, handle_cli_univar_exec,
        "Return a universal variable's value", univar_usage, NULL };


int unload_module(void)
{
	int res;
	struct cacheservers *cs, *cr;

	STANDARD_HANGUP_LOCALUSERS;
	ast_cli_unregister(&cli_univar);
	res = ast_unregister_application(app_getunivar);
	res |= ast_unregister_application(app_setunivar);
	res |= ast_unregister_application(app_delunivar);

	remove_cacheservers(caching_servers);	

	return res;
}

static int load_cacheservers(char *section)
{
        char *cfgvar=NULL;
        struct ast_config *cfg;
        int res = 0;
        struct cacheservers *newserver, *prev;
        char *newvar;
        char *slist;

        cfg = ast_load(UNIVERSAL_CONFIG);

        if (cfg) {
                if ((cfgvar = ast_variable_retrieve(cfg, section, "servers"))) {
                        ast_log(LOG_NOTICE, "Using section [%s]\n",section);
                        prev = NULL;
                        slist = strsep(&cfgvar, " ");
                        while(slist) {
                                newvar = strsep(&slist, ":");
                                if (newvar) {
                                        newserver = malloc(sizeof(struct cacheservers));
                                        if (newserver) {
                                                memset(newserver, 0, sizeof(struct cacheservers));
                                                newserver->next = NULL;
                                                strncpy(newserver->server, newvar, sizeof(newserver->server) - 1);
                                                strncpy(newserver->port, slist, sizeof(newserver->port) - 1);
                                                if (!prev) {
                                                        caching_servers =  newserver;
                                                }
                                                else {
                                                        prev->next = newserver;
                                                }
                                                prev=newserver;
                                        }
                                }
                                ast_log(LOG_NOTICE, "Added Caching server %s port %s\n",newserver->server,newserver->port);
                                slist = strsep(&cfgvar, " ");
                        }
                } else
                	res = -1;

		if ((cfgvar = ast_variable_retrieve(cfg, section, "allowreload"))) {
			if (!strcasecmp(cfgvar, "yes")) {
				allowreload=1;
			} else {
				allowreload=0;
			}
		}

                ast_destroy(cfg);
        }
        else
                res = -1;

        if (res) {
                ast_log(LOG_NOTICE, "Removing cache servers\n");
                remove_cacheservers(caching_servers);
        }
        return res;
}


int load_module(void)
{
	int res;
	
	res = ast_register_application(app_getunivar, getunivar_exec, getunivar_synopsis, getunivar_descrip);
	res |= ast_register_application(app_setunivar, setunivar_exec, setunivar_synopsis, setunivar_descrip);
        res |= ast_register_application(app_delunivar, delunivar_exec, delunivar_synopsis, delunivar_descrip);
	res |= ast_cli_register(&cli_univar);
	res |= load_cacheservers("general"); 

	return res;
}

int reload(void)
{
	/* This is dangerous, since we remove all the   */
	/* caching servers then add them again, any     */
	/* queries between the two will return errors.  */
	/* Added allowreload=yes option to config file  */
	/* without it reloads cannot be done. Only a    */
	/* restart or unload/load of the module will    */
	/* update from configs.                         */	

	if (allowreload) {
		remove_cacheservers(caching_servers);
		return (load_cacheservers("general"));
	} else {
		ast_verbose(VERBOSE_PREFIX_4 "Reloading disabled in config file\n");
	}

	return 0;
}



char *description(void)
{
	return tdesc;
}

int usecount(void)
{
	int res;
	STANDARD_USECOUNT(res);
	return res;
}

char *key()
{
	return AEFIRION_GPL_KEY;
}
