/*
 Created by Andy Powell

 Header for caching servers


*/

/* Maximum length of any variable */
#define MAXRESULT       1024
#define MAX_VARLEN      256
#define UNIVERSAL_CONFIG "caching.conf"


struct cacheservers {
        char server[80];
        char port[10];
        struct cacheservers *next;
};

struct cacheservers *caching_servers=NULL;

int remove_cacheservers()
{

        struct cacheservers *cs, *cr;
	char servername[80];
	char serverport[10];


        /* Remove all the servers */

        cs = caching_servers;

        while(cs) {
                cr = cs;
		strncpy(servername,cr->server,sizeof(servername));
		strncpy(serverport,cr->port,sizeof(serverport));
                cs = cs->next;
                free(cr);
		ast_log(LOG_NOTICE, "Removed caching server %s, port %s\n",servername,serverport);
        }

        return 0;

}


static int connect_all(struct memcache *mcc, struct cacheservers *caches)
{
        struct cacheservers *prev, *cr;
        int res = -1;
        int tres = 0;
        int cnt = 0;
        prev = caches;

        while(prev) {
                cr = prev;
                prev = prev->next;
                tres = tres + mc_server_add(mcc,  cr->server, cr->port);
                cnt++;
        }

        if (tres < cnt)
                res = 0;
        else
                res = 1;

        return res;

}

static int keyexists(struct cacheservers *caches, char *skey)
{
	struct memcache *mc = NULL;
        struct memcache_req *req;
        struct memcache_res *res1;
	int res = 1;

        mc = mc_new();
        if (mc == NULL) {
                ast_log(LOG_ERROR, "Can't allocate cache!\n");
                res = 1;
        } else {
		if (connect_all(mc,caches) == 0) {
	        	req = mc_req_new();
		        res1 = mc_req_add(req, skey, strlen(skey));
        		mc_get(mc, req);

		        if (res1->size <= 0) 
				res = 1;
			else 
				res = 0;

			mc_req_free(req);
			mc_server_disconnect_all(mc);
		} else {
                        ast_log(LOG_ERROR, "Can't connect!\n");
                        res = 1;
		}
		mc_free(mc);
	}
	return res;
}

void get_cachevar(struct cacheservers *caches, char *chresult, char *skey)
{
        struct memcache *mc = NULL;

        struct memcache_req *req;
        struct memcache_res *res1;
	int res = 0;
        char strval[MAX_VARLEN] = "";

        memset(strval, 0, sizeof(strval));

        mc = mc_new();
        if (mc == NULL) {
                ast_log(LOG_ERROR, "Can't allocate cache!\n");
                res = -1;
        } else {
                if (connect_all(mc,caches) == 0) {
        		req = mc_req_new();
		        res1 = mc_req_add(req, skey, strlen(skey));
        		mc_get(mc, req);
	
		        if (res1->size <= 0) {
        		        strncpy(chresult,"\0",sizeof(chresult));
		        } else {
        		        if (res1->size > sizeof(strval)) {
                		        strncpy(strval,res1->val,sizeof(strval));
	                	} else {
        	                	strncpy(strval,res1->val,res1->size);
	                	}
				memset(chresult, 0, sizeof(chresult));
		                snprintf(chresult,MAX_VARLEN-1,strval);
        		}

			mc_req_free(req);
			mc_server_disconnect_all(mc);
		} else {
                	ast_log(LOG_ERROR, "Can't connect!\n");
			res = -1;
		}
		mc_free(mc);

	}
}



static int set_cachevar(struct cacheservers *caches, char *varname, char *newvalue, int expires)
{
        struct memcache *mc = NULL;

        struct memcache_req *req;
        struct memcache_res *res1;
	int i = 0;
        int res = 0;
        int modok = 0;

        mc = mc_new();
        if (mc == NULL) {
                ast_log(LOG_ERROR, "Can't allocate cache!\n");
                res = -1;
        } else {
        	if (connect_all(mc,caches) == 0) {

			for (i=0;i<10;i++) {
				modok = (mc_set(mc, varname, strlen(varname), newvalue, strlen(newvalue), expires, 0) == 0);
		                if (modok) {
        		                req = mc_req_new();
                		        res1 = mc_req_add(req, varname, strlen(varname));
	                	        mc_req_free(req);
        	                	res = 0;
					break;
		                } else {
					sleep(0.8);
                		        ast_log(LOG_ERROR, "Try: %d :: Oh crap! Failed to add/modify variable %s to %s\n",i,varname,newvalue);
		               }
			}
                	mc_server_disconnect_all(mc);
	        } else {
        	        ast_log(LOG_ERROR, "Can't connect to anything!\n");
                	res = -1;
	        }
		 mc_free(mc);
	}
        return res;
}

static int del_cachevar(struct cacheservers *caches, char *varname)
{
	struct memcache *mc = NULL;
        struct memcache_req *req;
        struct memcache_res *res1;

        int res = 0;

	mc = mc_new();
        if (mc == NULL) {
        	ast_log(LOG_ERROR, "Can't allocate cache!\n");
                res = 1;
        } else {
		if (connect_all(mc, caches) == 0 ) {
        		mc_delete(mc, varname, strlen(varname), 0);
	                mc_server_disconnect_all(mc);
        	        res = 0;
	         } else {
		         ast_log(LOG_ERROR, "Can't connect!\n");
	                 res = 1;
        	 }
	mc_free(mc);
	}
	return res;
}


